"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.CreeveyManager = void 0;

var _coreEvents = require("@storybook/core-events");

var _shared = require("../../shared");

var _types = require("../../types");

var _creeveyClientApi = require("../shared/creeveyClientApi");

var _helpers = require("../shared/helpers");

var _register = require("./register");

var _utils = require("./utils");

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class CreeveyManager {
  constructor(storybookApi) {
    this.storybookApi = storybookApi;

    _defineProperty(this, "storyId", '');

    _defineProperty(this, "activeBrowser", '');

    _defineProperty(this, "selectedTestId", '');

    _defineProperty(this, "status", {
      isRunning: false,
      tests: {},
      browsers: []
    });

    _defineProperty(this, "creeveyApi", null);

    _defineProperty(this, "stories", {});

    _defineProperty(this, "updateStatusListeners", []);

    _defineProperty(this, "changeTestListeners", []);

    _defineProperty(this, "initAll", async () => {
      this.storybookApi.on(_coreEvents.STORY_RENDERED, this.onStoryRendered);
      this.storybookApi.on(_coreEvents.SET_STORIES, this.onSetStories);
      this.creeveyApi = await (0, _creeveyClientApi.initCreeveyClientApi)();
      this.creeveyApi.onUpdate(this.handleCreeveyUpdate);
      this.status = await this.creeveyApi.status;
    });

    _defineProperty(this, "handleCreeveyUpdate", update => {
      const {
        tests,
        removedTests = [],
        isRunning
      } = update;

      if ((0, _types.isDefined)(isRunning)) {
        this.status.isRunning = isRunning;
      }

      if ((0, _types.isDefined)(tests)) {
        const prevTests = this.status.tests;
        const prevStories = this.stories || {};
        Object.values(tests).filter(_types.isDefined).forEach(update => {
          const {
            id,
            skip,
            status,
            results,
            approved,
            storyId
          } = update;
          const test = prevTests[id];

          if (!test) {
            return prevTests[id] = update;
          }

          if ((0, _types.isDefined)(skip)) test.skip = skip;

          if ((0, _types.isDefined)(status)) {
            test.status = status;

            if ((0, _types.isDefined)(storyId) && (0, _types.isDefined)(prevStories[storyId])) {
              const story = prevStories[storyId];
              const storyStatus = this.getStoryTests(storyId);
              const oldStatus = storyStatus.map(x => x.id === id ? status : x.status).reduce((oldStatus, newStatus) => (0, _helpers.calcStatus)(oldStatus, newStatus), undefined);
              story.name = this.addStatusToStoryName(story.name, (0, _helpers.calcStatus)(oldStatus, status), skip || false);
            }
          }

          if ((0, _types.isDefined)(results)) test.results ? test.results.push(...results) : test.results = results;

          if ((0, _types.isDefined)(approved)) {
            Object.entries(approved).forEach(_ref => {
              let [image, retry] = _ref;
              return retry !== undefined && test && ((test.approved = (test === null || test === void 0 ? void 0 : test.approved) || {})[image] = retry);
            });
          }
        });
        removedTests.forEach(_ref2 => {
          let {
            id
          } = _ref2;
          return delete prevTests[id];
        });
        this.status.tests = prevTests;
        this.stories = prevStories;
        this.setPanelsTitle();
        void this.storybookApi.setStories(this.stories);
      }

      this.updateStatusListeners.forEach(x => x(update));
    });

    _defineProperty(this, "getCurrentTest", () => {
      return this.status.tests[this.selectedTestId];
    });

    _defineProperty(this, "onStoryRendered", storyId => {
      if (this.storyId === '') void this.addStatusesToSideBar();

      if (this.storyId !== storyId) {
        var _this$getTestsByStory, _this$getTestsByStory2;

        this.storyId = storyId;
        this.selectedTestId = (_this$getTestsByStory = (_this$getTestsByStory2 = this.getTestsByStoryIdAndBrowser(this.activeBrowser)[0]) === null || _this$getTestsByStory2 === void 0 ? void 0 : _this$getTestsByStory2.id) !== null && _this$getTestsByStory !== void 0 ? _this$getTestsByStory : '';
        this.setPanelsTitle();
        this.changeTestListeners.forEach(x => x(this.selectedTestId));
      }
    });

    _defineProperty(this, "onStart", () => {
      var _this$creeveyApi;

      (_this$creeveyApi = this.creeveyApi) === null || _this$creeveyApi === void 0 ? void 0 : _this$creeveyApi.start([this.selectedTestId]);
    });

    _defineProperty(this, "onStop", () => {
      var _this$creeveyApi2;

      (_this$creeveyApi2 = this.creeveyApi) === null || _this$creeveyApi2 === void 0 ? void 0 : _this$creeveyApi2.stop();
    });

    _defineProperty(this, "onImageApprove", (id, retry, image) => {
      var _this$creeveyApi3;

      return (_this$creeveyApi3 = this.creeveyApi) === null || _this$creeveyApi3 === void 0 ? void 0 : _this$creeveyApi3.approve(id, retry, image);
    });

    _defineProperty(this, "onStartAllStoryTests", () => {
      var _this$creeveyApi4;

      const ids = Object.values(this.status.tests).filter(_types.isDefined).filter(x => x.storyId === this.storyId).map(x => x.id);
      (_this$creeveyApi4 = this.creeveyApi) === null || _this$creeveyApi4 === void 0 ? void 0 : _this$creeveyApi4.start(ids);
    });

    _defineProperty(this, "onStartAllTests", () => {
      var _this$creeveyApi5;

      const ids = Object.values(this.status.tests).filter(_types.isDefined).map(x => x.id);
      (_this$creeveyApi5 = this.creeveyApi) === null || _this$creeveyApi5 === void 0 ? void 0 : _this$creeveyApi5.start(ids);
    });

    _defineProperty(this, "onSetStories", data => {
      const stories = data.v ? (0, _shared.denormalizeStoryParameters)(data) : data.stories;
      this.stories = stories;
    });

    _defineProperty(this, "setActiveBrowser", browser => {
      var _this$getTestsByStory3, _this$getTestsByStory4;

      this.activeBrowser = browser;
      this.selectedTestId = (_this$getTestsByStory3 = (_this$getTestsByStory4 = this.getTestsByStoryIdAndBrowser(this.activeBrowser)[0]) === null || _this$getTestsByStory4 === void 0 ? void 0 : _this$getTestsByStory4.id) !== null && _this$getTestsByStory3 !== void 0 ? _this$getTestsByStory3 : '';
      this.changeTestListeners.forEach(x => x(this.selectedTestId));
    });

    _defineProperty(this, "setSelectedTestId", testId => {
      this.selectedTestId = testId;
      this.changeTestListeners.forEach(x => x(this.selectedTestId));
    });

    _defineProperty(this, "getStoryTests", storyId => {
      if (!this.status || !this.status.tests) return [];
      return Object.values(this.status.tests).filter(result => (result === null || result === void 0 ? void 0 : result.storyId) === storyId).filter(_types.isDefined);
    });

    _defineProperty(this, "getBrowsers", () => {
      return this.status.browsers;
    });

    _defineProperty(this, "getTestsByStoryIdAndBrowser", browser => {
      return Object.values(this.status.tests).filter(x => (x === null || x === void 0 ? void 0 : x.browser) === browser && x.storyId === this.storyId).filter(_types.isDefined);
    });

    _defineProperty(this, "getTabTitle", browser => {
      const tests = this.getTestsByStoryIdAndBrowser(browser);
      const browserStatus = tests.map(x => x && x.status).reduce((oldStatus, newStatus) => (0, _helpers.calcStatus)(oldStatus, newStatus), undefined);
      const browserSkip = tests.length > 0 ? tests.every(x => x && x.skip) : false;
      const emojiStatus = (0, _utils.getEmojiByTestStatus)(browserStatus, browserSkip);
      return "".concat(emojiStatus ? "".concat(emojiStatus, " ") : '', "Creevey/").concat(browser);
    });

    _defineProperty(this, "setPanelsTitle", () => {
      var _this$storybookApi;

      const panels = (_this$storybookApi = this.storybookApi) === null || _this$storybookApi === void 0 ? void 0 : _this$storybookApi.getPanels();
      if (!panels) return;
      let firstPanelBrowser = this.activeBrowser;

      for (const p in panels) {
        var _panel$id;

        const panel = panels[p];

        if (((_panel$id = panel.id) === null || _panel$id === void 0 ? void 0 : _panel$id.indexOf(_register.ADDON_ID)) === 0 && panel.paramKey) {
          panel.title = this.getTabTitle(panel.paramKey);
          if (!firstPanelBrowser) firstPanelBrowser = panel.paramKey;
        }
      }

      this.storybookApi.setSelectedPanel("".concat(_register.ADDON_ID, "/panel/").concat(firstPanelBrowser));
    });

    this.storybookApi = storybookApi;
  }

  onUpdateStatus(listener) {
    this.updateStatusListeners.push(listener);
    return () => void (this.updateStatusListeners = this.updateStatusListeners.filter(x => x != listener));
  }

  onChangeTest(listener) {
    this.changeTestListeners.push(listener);
    return () => void (this.changeTestListeners = this.changeTestListeners.filter(x => x != listener));
  }

  async addStatusesToSideBar() {
    if (!Object.keys(this.stories).length) return;
    const stories = this.stories;
    Object.keys(this.stories).forEach(storyId => {
      const storyStatus = this.getStoryTests(storyId);
      const status = storyStatus.map(x => x.status).reduce((oldStatus, newStatus) => (0, _helpers.calcStatus)(oldStatus, newStatus), undefined);
      const skip = storyStatus.length > 0 ? storyStatus.every(x => x.skip) : false;
      this.stories[storyId].name = this.addStatusToStoryName(stories[storyId].name, status, skip);
    });
    await this.storybookApi.setStories(this.stories);
  }

  addStatusToStoryName(name, status, skip) {
    name = name.replace(/^(❌|✔|🟡|🕗|⏸) /, '');
    const emojiStatus = (0, _utils.getEmojiByTestStatus)(status, skip);
    return "".concat(emojiStatus ? "".concat(emojiStatus, " ") : '', " ").concat(name);
  }

}

exports.CreeveyManager = CreeveyManager;