"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ResultsPage = void 0;
exports.ResultsPageInternal = ResultsPageInternal;

var _react = _interopRequireWildcard(require("react"));

var _ImagesView = require("./ImagesView/ImagesView");

var _PageHeader = require("./PageHeader/PageHeader");

var _PageFooter = require("./PageFooter/PageFooter");

var _helpers = require("../helpers");

var _theming = require("@storybook/theming");

var _components = require("@storybook/components");

var _viewMode = require("../viewMode");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const Wrapper = _theming.styled.div({
  width: '100%',
  height: '100%',
  boxSizing: 'border-box',
  display: 'flex',
  flexDirection: 'column'
});

const ImagesViewContainer = _theming.styled.div(_ref => {
  let {
    theme
  } = _ref;
  return {
    background: theme.base == 'light' ? theme.color.mediumlight : theme.color.darker,
    flexGrow: 1,
    padding: '20px 0'
  };
});

const HeaderContainer = _theming.styled.div({
  position: 'sticky',
  top: 0,
  zIndex: 1
});

const BodyContainer = _theming.styled.div({
  flexGrow: 1
});

const FooterContainer = _theming.styled.div({
  position: 'sticky',
  bottom: 0,
  zIndex: 1
});

const Container = _theming.styled.div(_ref2 => {
  let {
    height = '100vh'
  } = _ref2;
  return {
    height,
    width: '100%',
    overflowY: 'hidden',
    display: 'flex',
    flexDirection: 'column'
  };
});

function ResultsPageInternal(_ref3) {
  var _results, _Object$keys$, _result$images, _result$images2;

  let {
    id,
    path,
    results = [],
    approved = {},
    theme,
    onImageApprove,
    showTitle = false,
    height
  } = _ref3;
  const [retry, setRetry] = (0, _react.useState)(results.length);
  const result = (_results = results[retry - 1]) !== null && _results !== void 0 ? _results : {};
  const [imageName, setImageName] = (0, _react.useState)((_Object$keys$ = Object.keys((_result$images = result.images) !== null && _result$images !== void 0 ? _result$images : {})[0]) !== null && _Object$keys$ !== void 0 ? _Object$keys$ : '');
  const [viewMode, setViewMode] = (0, _react.useState)((0, _viewMode.getViewMode)());
  (0, _react.useEffect)(() => setRetry(results.length), [results.length]);
  const url = (0, _helpers.getImageUrl)(path, imageName);
  const image = (_result$images2 = result.images) === null || _result$images2 === void 0 ? void 0 : _result$images2[imageName];
  const canApprove = Boolean(image && approved[imageName] != retry - 1 && result.status != 'success');
  const hasDiffAndExpect = canApprove && Boolean((image === null || image === void 0 ? void 0 : image.diff) && image.expect);
  const imagesWithError = result.images ? Object.keys(result.images).filter(imageName => {
    var _imageName;

    return result.status != 'success' && approved[imageName] != retry - 1 && ((_imageName = (result.images || {})[imageName]) === null || _imageName === void 0 ? void 0 : _imageName.error) != null;
  }) : [];

  const handleApprove = () => onImageApprove(id, retry - 1, imageName);

  const handleChangeViewMode = mode => {
    localStorage.setItem(_viewMode.VIEW_MODE_KEY, mode);
    setViewMode(mode);
  };

  return /*#__PURE__*/_react.default.createElement(Container, {
    height: height
  }, /*#__PURE__*/_react.default.createElement(HeaderContainer, null, /*#__PURE__*/_react.default.createElement(_PageHeader.PageHeader, {
    title: path,
    images: result.images,
    errorMessage: result.error,
    showViewModes: hasDiffAndExpect,
    viewMode: viewMode,
    onViewModeChange: handleChangeViewMode,
    onImageChange: setImageName,
    showTitle: showTitle,
    imagesWithError: imagesWithError
  })), /*#__PURE__*/_react.default.createElement(BodyContainer, null, /*#__PURE__*/_react.default.createElement(_components.ScrollArea, {
    vertical: true
  }, /*#__PURE__*/_react.default.createElement(Wrapper, null, /*#__PURE__*/_react.default.createElement(ImagesViewContainer, {
    theme: theme
  }, image ? /*#__PURE__*/_react.default.createElement(_ImagesView.ImagesView, {
    url: url,
    image: image,
    canApprove: canApprove,
    mode: viewMode
  }) : /*#__PURE__*/_react.default.createElement(_components.Placeholder, null, "Image ".concat(imageName, " not found")))))), results.length ? /*#__PURE__*/_react.default.createElement(FooterContainer, null, /*#__PURE__*/_react.default.createElement(_PageFooter.PageFooter, {
    canApprove: canApprove,
    retry: retry,
    retriesCount: results.length,
    onRetryChange: setRetry,
    onApprove: handleApprove
  })) : null);
}

const ResultsPage = (0, _theming.withTheme)(ResultsPageInternal);
exports.ResultsPage = ResultsPage;