"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _default;

var _cluster = _interopRequireDefault(require("cluster"));

var _config = require("./config");

var _logger = require("./logger");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

// NOTE: Impure function, mutate config by adding gridUrl prop
async function startWebdriverServer(config, options) {
  if (config.useDocker) {
    return (await Promise.resolve().then(() => _interopRequireWildcard(require('./docker')))).default(config, options.browser, async () => (await Promise.resolve().then(() => _interopRequireWildcard(require('./selenium/selenoid')))).startSelenoidContainer(config, options.debug));
  } else {
    return (await Promise.resolve().then(() => _interopRequireWildcard(require('./selenium/selenoid')))).startSelenoidStandalone(config, options.debug);
  }
}

async function _default(options) {
  const config = await (0, _config.readConfig)(options);
  const {
    browser = _config.defaultBrowser,
    extract,
    tests,
    update,
    webpack,
    ui,
    port
  } = options;
  if (!config) return; // NOTE: We don't need docker nor selenoid for webpack or update options

  if (!(config.gridUrl || Object.values(config.browsers).every(({
    gridUrl
  }) => gridUrl)) && !extract && !webpack && !tests && !update) {
    await startWebdriverServer(config, options);
  }

  switch (true) {
    case Boolean(extract) || tests:
      {
        return (await Promise.resolve().then(() => _interopRequireWildcard(require('./extract')))).default(config, options);
      }

    case Boolean(update):
      {
        return (await Promise.resolve().then(() => _interopRequireWildcard(require('./update')))).default(config, typeof update == 'string' ? update : undefined);
      }

    case webpack:
      {
        _logger.logger.info('Starting Webpack Compiler');

        return (await Promise.resolve().then(() => _interopRequireWildcard(require('./loaders/webpack/compile')))).default(config, options);
      }

    case _cluster.default.isPrimary:
      {
        _logger.logger.info('Starting Master Process');

        const resolveApi = (await Promise.resolve().then(() => _interopRequireWildcard(require('./master/server')))).default(config.reportDir, port, ui);
        return (await Promise.resolve().then(() => _interopRequireWildcard(require('./master')))).default(config, options, resolveApi);
      }

    default:
      {
        _logger.logger.info(`Starting Worker for ${browser}`);

        return (await Promise.resolve().then(() => _interopRequireWildcard(require('./worker')))).default(config, { ...options,
          browser
        });
      }
  }
}