"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = master;

var _path = _interopRequireDefault(require("path"));

var _types = require("../../types");

var _stories = require("../stories");

var _runner = _interopRequireDefault(require("./runner"));

var _start = require("../loaders/webpack/start");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function mergeTests(testsWithReports, testsFromStories) {
  Object.values(testsFromStories).filter(_types.isDefined).forEach(test => {
    const testWithReport = testsWithReports[test.id];
    if (!testWithReport) return;
    test.retries = testWithReport.retries;
    if (testWithReport.status == 'success' || testWithReport.status == 'failed') test.status = testWithReport.status;
    test.results = testWithReport.results;
    test.approved = testWithReport.approved;
  });
  return testsFromStories;
}

async function master(config, options) {
  if (config.useWebpackToExtractTests) await (0, _start.startWebpackCompiler)();
  const runner = new _runner.default(config);

  const reportDataPath = _path.default.join(config.reportDir, 'data.js');

  let testsFromReport = {};

  try {
    testsFromReport = await Promise.resolve(`${reportDataPath}`).then(s => _interopRequireWildcard(require(s)));
  } catch (error) {// Ignore error
  }

  await runner.init();
  const tests = await (0, _stories.loadTestsFromStories)(Object.keys(config.browsers), listener => config.storiesProvider(config, options, listener), testsDiff => {
    runner.updateTests(testsDiff);
    (0, _stories.saveTestsJson)(runner.tests, config.reportDir);
  });
  runner.tests = mergeTests(testsFromReport, tests);
  (0, _stories.saveTestsJson)(runner.tests, config.reportDir);
  return runner;
}