"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getStorybookFramework = getStorybookFramework;
exports.getStorybookVersion = getStorybookVersion;
exports.hasDocsAddon = hasDocsAddon;
exports.hasSvelteCSFAddon = hasSvelteCSFAddon;
exports.importStorybookClientLogger = void 0;
exports.importStorybookConfig = importStorybookConfig;
exports.importStorybookCoreEvents = exports.importStorybookCoreCommon = void 0;
exports.isCSFv3Enabled = isCSFv3Enabled;
exports.isStorybookVersion = isStorybookVersion;
exports.isStorybookVersionGreaterThan = isStorybookVersionGreaterThan;
exports.isStorybookVersionLessThan = isStorybookVersionLessThan;
exports.storybookDirRef = exports.storybookConfigRef = exports.resolveFromStorybookCoreServer = exports.resolveFromStorybookCoreClient = exports.resolveFromStorybookBuilderWebpack4 = exports.resolveFromStorybookAddonDocs = exports.resolveFromStorybook = void 0;

var _path = _interopRequireDefault(require("path"));

var _resolveFrom = _interopRequireDefault(require("resolve-from"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const supportedFrameworks = ['react', 'vue', 'vue3', 'angular', 'marionette', 'mithril', 'marko', 'html', 'svelte', 'riot', 'ember', 'preact', 'rax', 'aurelia', 'server', 'web-components'];
const storybookDirRef = {
  current: _path.default.resolve('.storybook')
};
exports.storybookDirRef = storybookDirRef;

const resolveFromStorybook = modulePath => (0, _resolveFrom.default)(storybookDirRef.current, modulePath);

exports.resolveFromStorybook = resolveFromStorybook;

const resolveFromStorybookAddonDocs = modulePath => (0, _resolveFrom.default)(resolveFromStorybook('@storybook/addon-docs'), modulePath);

exports.resolveFromStorybookAddonDocs = resolveFromStorybookAddonDocs;

const resolveFromStorybookBuilderWebpack4 = modulePath => (0, _resolveFrom.default)(resolveFromStorybook('@storybook/builder-webpack4'), modulePath);

exports.resolveFromStorybookBuilderWebpack4 = resolveFromStorybookBuilderWebpack4;

const resolveFromStorybookCoreClient = modulePath => (0, _resolveFrom.default)(resolveFromStorybook('@storybook/core-client'), modulePath);

exports.resolveFromStorybookCoreClient = resolveFromStorybookCoreClient;

const resolveFromStorybookCoreServer = modulePath => (0, _resolveFrom.default)(resolveFromStorybook('@storybook/core-server'), modulePath);

exports.resolveFromStorybookCoreServer = resolveFromStorybookCoreServer;

const importFromStorybook = modulePath => Promise.resolve(`${resolveFromStorybook(modulePath)}`).then(s => _interopRequireWildcard(require(s)));

const importStorybookClientLogger = () => importFromStorybook('@storybook/client-logger');

exports.importStorybookClientLogger = importStorybookClientLogger;

const importStorybookCoreCommon = () => importFromStorybook('@storybook/core-common');

exports.importStorybookCoreCommon = importStorybookCoreCommon;

const importStorybookCoreEvents = () => importFromStorybook('@storybook/core-events');

exports.importStorybookCoreEvents = importStorybookCoreEvents;

function hasDocsAddon() {
  try {
    resolveFromStorybook('@storybook/addon-docs');
    return true;
  } catch (_) {
    return false;
  }
}

function hasSvelteCSFAddon() {
  try {
    resolveFromStorybook('@storybook/addon-svelte-csf');
    return true;
  } catch (_) {
    return false;
  }
}

function getStorybookVersion() {
  // eslint-disable-next-line @typescript-eslint/no-var-requires
  const {
    version
  } = require(resolveFromStorybook('@storybook/core/package.json'));

  return version;
}

function isStorybookVersionLessThan(major, minor) {
  const [sbMajor, sbMinor] = (process.env.__CREEVEY_STORYBOOK_VERSION__ ?? getStorybookVersion()).split('.');
  return Number(sbMajor) < major || minor != undefined && Number(sbMajor) == major && Number(sbMinor) < minor;
}

function isStorybookVersionGreaterThan(major, minor) {
  const [sbMajor, sbMinor] = (process.env.__CREEVEY_STORYBOOK_VERSION__ ?? getStorybookVersion()).split('.');
  return Number(sbMajor) > major || minor != undefined && Number(sbMajor) == major && Number(sbMinor) > minor;
}

function isStorybookVersion(major, minor) {
  const [sbMajor, sbMinor] = (process.env.__CREEVEY_STORYBOOK_VERSION__ ?? getStorybookVersion()).split('.');
  return Number(sbMajor) == major || minor != undefined && Number(sbMajor) == major && Number(sbMinor) == minor;
}

function getStorybookFramework() {
  const framework = process.env.__CREEVEY_STORYBOOK_FRAMEWORK__ ?? supportedFrameworks.find(framework => {
    try {
      return require.resolve(resolveFromStorybook(`@storybook/${framework}`));
    } catch (_) {
      return false;
    }
  });
  if (!framework) throw new Error("Couldn't detect used Storybook framework. Please ensure that you install `@storybook/<framework>` package");
  return framework;
}

const storybookConfigRef = {
  current: {
    stories: []
  }
};
exports.storybookConfigRef = storybookConfigRef;

async function importStorybookConfig() {
  const configPath = `${storybookDirRef.current}/main`;

  try {
    return storybookConfigRef.current = (await Promise.resolve(`${require.resolve(configPath)}`).then(s => _interopRequireWildcard(require(s)))).default;
  } catch (_) {
    const storybookUtilsPath = '@storybook/core-common/dist/cjs/utils';
    const serverRequireModule = 'interpret-require'; // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment

    const {
      getInterpretedFile
    } = await Promise.resolve(`${resolveFromStorybook(`${storybookUtilsPath}/interpret-files`)}`).then(s => _interopRequireWildcard(require(s))); // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment

    const {
      default: serverRequireFallback,
      serverRequire = serverRequireFallback
    } = await Promise.resolve(`${resolveFromStorybook(`${storybookUtilsPath}/${serverRequireModule}`)}`).then(s => _interopRequireWildcard(require(s))); // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-call

    const mainConfigFile = getInterpretedFile(configPath); // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-call

    return storybookConfigRef.current = serverRequire(mainConfigFile);
  }
}

async function isCSFv3Enabled() {
  var _await$importStoryboo, _await$importStoryboo2;

  return ((_await$importStoryboo = await importStorybookConfig()) === null || _await$importStoryboo === void 0 ? void 0 : (_await$importStoryboo2 = _await$importStoryboo.features) === null || _await$importStoryboo2 === void 0 ? void 0 : _await$importStoryboo2.previewCsfV3) ?? false;
}