"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.extractStoriesData = extractStoriesData;
exports.loadStories = void 0;

var _path = _interopRequireDefault(require("path"));

var _cluster = _interopRequireDefault(require("cluster"));

var _chokidar = _interopRequireDefault(require("chokidar"));

var _types = require("../../../types");

var _utils = require("../../utils");

var _messages = require("../../messages");

var _helpers = require("../helpers");

var _logger = require("../../logger");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

async function initStorybookEnvironment() {
  // @ts-expect-error There is no @types/global-jsdom package
  (await Promise.resolve().then(() => _interopRequireWildcard(require('global-jsdom')))).default(undefined, {
    url: 'http://localhost'
  }); // NOTE Cutoff `jsdom` part from userAgent, because storybook check enviroment and create events channel if runs in browser
  // https://github.com/storybookjs/storybook/blob/v5.2.8/lib/core/src/client/preview/start.js#L98
  // Example: "Mozilla/5.0 (linux) AppleWebKit/537.36 (KHTML, like Gecko) jsdom/15.2.1"

  Object.defineProperty(window.navigator, 'userAgent', {
    value: window.navigator.userAgent.split(' ').filter(token => !token.startsWith('jsdom')).join(' ')
  }); // TODO Look at creevey debug flag

  const {
    logger
  } = await (0, _helpers.importStorybookClientLogger)(); // NOTE: Disable duplication warnings for >=6.2 storybook

  if (_cluster.default.isWorker) logger.warn = _types.noop; // NOTE: disable logger for 5.x storybook

  logger.debug = _types.noop;
  return Promise.resolve().then(() => _interopRequireWildcard(require('../entry')));
}

function watchStories(channel, watcher, initialFiles) {
  const watchingFiles = initialFiles;
  let storiesByFiles = new Map();
  (0, _messages.subscribeOn)('shutdown', () => void watcher.close());
  watcher.add(Array.from(watchingFiles));
  watcher.on('change', filePath => storiesByFiles.set(_path.default.isAbsolute(filePath) ? filePath : `./${filePath.replace(/\\/g, '/')}`, []));
  watcher.on('unlink', filePath => storiesByFiles.set(_path.default.isAbsolute(filePath) ? filePath : `./${filePath.replace(/\\/g, '/')}`, []));
  return data => {
    const stories = data.stories;
    const files = new Set(Object.values(stories).map(story => story.parameters.fileName));
    const addedFiles = Array.from(files).filter(filePath => !watchingFiles.has(filePath));
    const removedFiles = Array.from(watchingFiles).filter(filePath => !files.has(filePath));
    watcher.add(addedFiles);
    addedFiles.forEach(filePath => {
      watchingFiles.add(filePath);
      storiesByFiles.set(filePath, []);
    });
    removedFiles.forEach(filePath => watchingFiles.delete(filePath));
    Object.values(stories).forEach(story => {
      var _storiesByFiles$get;

      return (_storiesByFiles$get = storiesByFiles.get(story.parameters.fileName)) === null || _storiesByFiles$get === void 0 ? void 0 : _storiesByFiles$get.push(story);
    });
    channel.emit('storiesUpdated', storiesByFiles);
    storiesByFiles = new Map();
  };
}

function loadStoriesFromBundle(watch) {
  const bundlePath = _path.default.join((0, _utils.getCreeveyCache)(), 'storybook/main.js');

  if (watch) {
    (0, _messages.subscribeOn)('webpack', message => {
      if (message.type != 'rebuild succeeded') return;
      Object.values(global.__CREEVEY_HMR_DATA__).filter(({
        callback
      }) => callback).forEach(({
        data,
        callback
      }) => callback(data));
      delete require.cache[bundlePath];
      Promise.resolve(`${bundlePath}`).then(s => _interopRequireWildcard(require(s)));
    });
  }

  Promise.resolve(`${bundlePath}`).then(s => _interopRequireWildcard(require(s)));
}

async function loadStoriesDirectly(config, {
  watcher,
  debug
}) {
  const {
    toRequireContext,
    normalizeStoriesEntry
  } = await (0, _helpers.importStorybookCoreCommon)();
  const {
    addParameters,
    configure
  } = await Promise.resolve().then(() => _interopRequireWildcard(require('../entry')));
  const requireContext = await (await Promise.resolve().then(() => _interopRequireWildcard(require('../../loaders/babel/register')))).default(config, debug);

  const preview = (() => {
    try {
      return require.resolve(`${config.storybookDir}/preview`);
    } catch (_) {
      /* noop */
    }
  })();

  const {
    stories
  } = await (0, _helpers.importStorybookConfig)();
  const contexts = stories.map(entry => {
    const normalizedEntry = normalizeStoriesEntry(entry, {
      configDir: config.storybookDir,
      workingDir: process.cwd()
    });
    const {
      path: storiesPath,
      recursive,
      match
    } = toRequireContext(normalizedEntry);
    watcher === null || watcher === void 0 ? void 0 : watcher.add(_path.default.resolve(config.storybookDir, storiesPath));
    return () => requireContext(storiesPath, recursive, new RegExp(match));
  });

  let disposeCallback = data => void data;

  Object.assign(module, {
    hot: {
      data: {},

      accept() {
        /* noop */
      },

      dispose(callback) {
        disposeCallback = callback;
      }

    }
  });

  async function startStorybook() {
    if (preview) {
      const {
        parameters,
        globals,
        globalTypes
      } = await Promise.resolve(`${preview}`).then(s => _interopRequireWildcard(require(s)));
      if (parameters) addParameters(parameters);
      if (globals) addParameters({
        globals
      });
      if (globalTypes) addParameters({
        globalTypes
      });
    }

    try {
      configure(contexts.map(ctx => ctx()), module, false);
    } catch (error) {
      if (_cluster.default.isPrimary) _logger.logger.error(error);
    }
  }

  watcher === null || watcher === void 0 ? void 0 : watcher.add(config.storybookDir);
  watcher === null || watcher === void 0 ? void 0 : watcher.on('all', (_event, filename) => {
    var _module$hot;

    disposeCallback((_module$hot = module.hot) === null || _module$hot === void 0 ? void 0 : _module$hot.data);
    delete require.cache[filename];
    void startStorybook();
  });
  void startStorybook();
} // TODO Do we need to support multiple storybooks here?


const loadStories = async (config, {
  watch,
  debug
}, storiesListener) => {
  const storybookApi = await initStorybookEnvironment();
  const Events = await (0, _helpers.importStorybookCoreEvents)();
  const {
    channel
  } = storybookApi;
  channel.removeAllListeners(Events.CURRENT_STORY_WAS_SET);
  channel.on('storiesUpdated', storiesListener);
  let watcher;
  if (watch) watcher = _chokidar.default.watch([], {
    ignoreInitial: true
  });
  const loadPromise = new Promise(resolve => {
    channel.once(Events.SET_STORIES, data => {
      const stories = data.stories;
      const files = new Set(Object.values(stories).map(story => story.parameters.fileName));
      if (watcher) channel.on(Events.SET_STORIES, watchStories(channel, watcher, files));
      resolve(stories);
    });
  });
  if (config.useWebpackToExtractTests) loadStoriesFromBundle(watch);else void loadStoriesDirectly(config, {
    watcher,
    debug
  });
  return loadPromise;
};

exports.loadStories = loadStories;

async function extractStoriesData(config, {
  watch,
  debug
}) {
  const storybookApi = await initStorybookEnvironment();
  const Events = await (0, _helpers.importStorybookCoreEvents)();
  const {
    channel
  } = storybookApi;
  channel.removeAllListeners(Events.CURRENT_STORY_WAS_SET);
  const loadPromise = new Promise(resolve => channel.once(Events.SET_STORIES, resolve));
  if (config.useWebpackToExtractTests) loadStoriesFromBundle(watch);else void loadStoriesDirectly(config, {
    debug
  });
  return loadPromise;
}