import React, { useState, useEffect } from 'react';
import { ImagesView } from './ImagesView/ImagesView';
import { PageHeader } from './PageHeader/PageHeader';
import { PageFooter } from './PageFooter/PageFooter';
import { getImageUrl } from '../helpers';
import { styled, withTheme } from '@storybook/theming';
import { Placeholder, ScrollArea } from '@storybook/components';
import { getViewMode, VIEW_MODE_KEY } from '../viewMode';
const Wrapper = styled.div({
  width: '100%',
  height: '100%',
  boxSizing: 'border-box',
  display: 'flex',
  flexDirection: 'column'
});
const ImagesViewContainer = styled.div(_ref => {
  let {
    theme
  } = _ref;
  return {
    background: theme.base == 'light' ? theme.color.mediumlight : theme.color.darker,
    flexGrow: 1,
    padding: '20px 0'
  };
});
const HeaderContainer = styled.div({
  position: 'sticky',
  top: 0,
  zIndex: 1
});
const BodyContainer = styled.div({
  flexGrow: 1
});
const FooterContainer = styled.div({
  position: 'sticky',
  bottom: 0,
  zIndex: 1
});
const Container = styled.div(_ref2 => {
  let {
    height = '100vh'
  } = _ref2;
  return {
    height,
    width: '100%',
    overflowY: 'hidden',
    display: 'flex',
    flexDirection: 'column'
  };
});
export function ResultsPageInternal(_ref3) {
  var _results, _Object$keys$, _result$images, _result$images2;

  let {
    id,
    path,
    results = [],
    approved = {},
    theme,
    onImageApprove,
    showTitle = false,
    height
  } = _ref3;
  const [retry, setRetry] = useState(results.length);
  const result = (_results = results[retry - 1]) !== null && _results !== void 0 ? _results : {};
  const [imageName, setImageName] = useState((_Object$keys$ = Object.keys((_result$images = result.images) !== null && _result$images !== void 0 ? _result$images : {})[0]) !== null && _Object$keys$ !== void 0 ? _Object$keys$ : '');
  const [viewMode, setViewMode] = useState(getViewMode());
  useEffect(() => setRetry(results.length), [results.length]);
  const url = getImageUrl(path, imageName);
  const image = (_result$images2 = result.images) === null || _result$images2 === void 0 ? void 0 : _result$images2[imageName];
  const canApprove = Boolean(image && approved[imageName] != retry - 1 && result.status != 'success');
  const hasDiffAndExpect = canApprove && Boolean((image === null || image === void 0 ? void 0 : image.diff) && image.expect);
  const imagesWithError = result.images ? Object.keys(result.images).filter(imageName => {
    var _imageName;

    return result.status != 'success' && approved[imageName] != retry - 1 && ((_imageName = (result.images || {})[imageName]) === null || _imageName === void 0 ? void 0 : _imageName.error) != null;
  }) : [];

  const handleApprove = () => onImageApprove(id, retry - 1, imageName);

  const handleChangeViewMode = mode => {
    localStorage.setItem(VIEW_MODE_KEY, mode);
    setViewMode(mode);
  };

  return /*#__PURE__*/React.createElement(Container, {
    height: height
  }, /*#__PURE__*/React.createElement(HeaderContainer, null, /*#__PURE__*/React.createElement(PageHeader, {
    title: path,
    images: result.images,
    errorMessage: result.error,
    showViewModes: hasDiffAndExpect,
    viewMode: viewMode,
    onViewModeChange: handleChangeViewMode,
    onImageChange: setImageName,
    showTitle: showTitle,
    imagesWithError: imagesWithError
  })), /*#__PURE__*/React.createElement(BodyContainer, null, /*#__PURE__*/React.createElement(ScrollArea, {
    vertical: true
  }, /*#__PURE__*/React.createElement(Wrapper, null, /*#__PURE__*/React.createElement(ImagesViewContainer, {
    theme: theme
  }, image ? /*#__PURE__*/React.createElement(ImagesView, {
    url: url,
    image: image,
    canApprove: canApprove,
    mode: viewMode
  }) : /*#__PURE__*/React.createElement(Placeholder, null, "Image ".concat(imageName, " not found")))))), results.length ? /*#__PURE__*/React.createElement(FooterContainer, null, /*#__PURE__*/React.createElement(PageFooter, {
    canApprove: canApprove,
    retry: retry,
    retriesCount: results.length,
    onRetryChange: setRetry,
    onApprove: handleApprove
  })) : null);
}
export const ResultsPage = withTheme(ResultsPageInternal);