import cluster from 'cluster';
import { isWebpackMessage } from '../../../types';
import { emitWebpackMessage } from '../../messages';
export function startWebpackCompiler() {
  return new Promise((resolve, reject) => {
    cluster.setupMaster({
      args: ['--webpack', ...process.argv.slice(2)]
    });
    const webpackCompiler = cluster.fork();
    webpackCompiler.on('message', message => {
      if (!isWebpackMessage(message)) return;
      Object.values(cluster.workers ?? {}).filter(worker => worker != webpackCompiler).forEach(worker => worker === null || worker === void 0 ? void 0 : worker.send(message));

      switch (message.type) {
        case 'success':
          return resolve();

        case 'fail':
          return reject();

        case 'rebuild succeeded':
        case 'rebuild failed':
          return emitWebpackMessage(message);
      }
    });
  });
}